/*
=====================================================================
	DRCSTR.CPP (Version 1.00)
		DRC's string function library source code.
	Copyright (c) 2002 by Daniel R. Collins. All rights reserved.
	Permission is granted to freely copy this document,
		and to alter it in any way for personal use.
=====================================================================
*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*
---------------------------------------------------------------------
	String basics
---------------------------------------------------------------------
*/

// Shifts a string up or down a given number of character places.
void strshift (char *s, int n)
	{
	if (n < 0)
		{
		if (-n >= (int)strlen(s)) memset(s, 0, 1);
		else memmove(s, s-n, strlen(s-n)+1);
		}
	else if (n > 0)
		{
		memmove(s+n, s, strlen(s)+1);
		memset(s, ' ', n);
		}
	}


// Shifts a string up and inserts specified text.
void strinsert (char *s, const char *src)
	{
	int length = strlen(src);
	strshift(s, length);
	memcpy(s, src, length);
	}


// Removes n characters and replaces with specified text.
void strreplace (char *s, const char *src, int n)
	{
	strshift(s, -n);
	strinsert(s, src);
	}


// Finds a substring and returns pointer right after it.
char *strskip (const char *s, const char *src)
	{
	char *ptr = strstr(s, src);
	if (ptr != NULL)
		ptr += strlen(src);
	return ptr;
	}


/*
---------------------------------------------------------------------
	String justification
---------------------------------------------------------------------
*/

// Pads on the left.
void strPadLeft (char *s, int n, char c)
	{
	if (n <= 0) return;
	strshift(s, n);
	memset(s, c, n);
	}


// Pads on the right.
void strPadRight (char *s, int n, char c)
	{
	if (n <= 0) return;
	int length = strlen(s);
	memset(s+length, c, n);
	s[length+n] = '\0';
	}


// Justifies in center (pad both sides).
void strJustCenter (char *s, int n, char c)
	{
	int extra = n-strlen(s);
	int left  = extra/2;
	int right = extra - left;

	strPadLeft(s, left, c);
	strPadRight(s, right, c);
	}


// Justifies on the left (pad on right).
void strJustLeft (char *s, int n, char c)
	{
	strPadRight(s, n-strlen(s), c);
	}


// Justifies on the right (pad on left).
void strJustRight (char *s, int n, char c)
	{
	strPadLeft(s, n-strlen(s), c);
	}


/*
---------------------------------------------------------------------
	String miscellany
---------------------------------------------------------------------
*/

// Converts entire string to lower case.
void strLowerCase (char *s)
	{
	while (*(s++) = tolower(*s));
	}


// Converts entire string to upper case.
void strUpperCase (char *s)
	{
	while (*(s++) = toupper(*s));
	}


// Converts integer to string with commas.
void strIntComma (char *s, long n)
	{
	sprintf(s, "%ld", n);
	char *p = s+strlen(s);
	while ((p -= 3) > s)
		if (isdigit(*(p-1)))
			strinsert(p, ",");
	}


/*
---------------------------------------------------------------------
	String tokenization
---------------------------------------------------------------------
*/

// Global token delimiter cache
static char g_cDelimit = '\t';


// Set token delimiter
void strTokDelim (char d)
	{
	g_cDelimit = d;
	}


// Given a string, return a pointer to the Nth token.
const char *strTokPtr (const char *s, int idx)
	{
	for (; *s && idx; ++s)
		if (*s == g_cDelimit && *(s+1) != g_cDelimit)
			--idx;
	return (*s ? s : NULL);
	}


// Given a string, count the tokens within it.
int strTokCount (const char *s)
	{
	for (int count = 1; *s; ++s)
		if (*s == g_cDelimit && *(s+1) != g_cDelimit)
			++count;
	return count;
	}


// Given a string, copy the Nth token.
void strTokCopy (const char *s, int idx, char *dest)
	{
	const char *pc;
	if (pc = strTokPtr(s, idx))
		while (*pc != '\0' && *pc != '\n' && *pc != g_cDelimit)
			*dest++ = *pc++;
	*dest = '\0';
	}


// Given a string, return the integer value of the Nth token.
int strTokValInt (const char *s, int idx)
	{
	const char *pc = strTokPtr(s, idx);
	return (pc ? atoi(pc) : 0);
	}


// Given a string, return the double value of the Nth token.
double strTokValDbl (const char *s, int idx)
	{
	const char *pc = strTokPtr(s, idx);
	return (pc ? atof(pc) : 0.0);
	}

