/*
==========================================================================
	MEMINST.CPP (Version 1.04)
		Source code for simple C/C++ memory instrumentation system.
    (Inspired by sample C++ code for program-defined delete.)
  Copyright (c) 2000-2004 Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
==========================================================================
*/

#include <assert.h>   // define assert()
#include <process.h>  // define abort()
#include <malloc.h>   // define malloc(), _msize()
#include <stdio.h>    // define printf(), etc.


// Global module variables
unsigned long gAllocBlocks = 0;  // count of allocated blocks
unsigned long gAllocBytes  = 0;  // count of allocated bytes


// Module messaging function
static void sendMsg (const char *msg) { printf("%s\n", msg); }


/*
--------------------------------------------------------------------------
  Program-defined operators "new" & 'delete"
--------------------------------------------------------------------------
*/

#ifndef DISABLE_MEMINST

void *operator new (size_t stAllocateBlock)
	{
	static int fInOpNew = 0;    // guard flag for re-entry

	if (!fInOpNew)
		{
		fInOpNew = 1;
		gAllocBlocks++;
		gAllocBytes += stAllocateBlock;
		fInOpNew = 0;
		}

	return malloc(stAllocateBlock);
	}


void operator delete (void *pvMem)
	{
	static int fInOpDelete = 0;    // guard flag for re-entry

	if (!fInOpDelete && pvMem != NULL)
		{
		fInOpDelete = 1;
		gAllocBlocks--;
		gAllocBytes -= _msize(pvMem);
		fInOpDelete = 0;
		}

	free(pvMem);
	}

#endif // DISABLE_MEMINST


/*
--------------------------------------------------------------------------
  Accessors to memory instrumentation info
--------------------------------------------------------------------------
*/

unsigned long memInstGetBlocks (void) {
  return gAllocBlocks;
}


unsigned long memInstGetBytes (void) {
  return gAllocBytes;
}


void memInstReport (void) {
	#ifndef DISABLE_MEMINST
		char buf[256];
		sprintf(buf, "MEMORY REPORT: Currently %lu memory block(s) for %lu byte(s).",
			gAllocBlocks, gAllocBytes);
		sendMsg(buf);
	#endif
}


/*
--------------------------------------------------------------------------
  New handlers to be registered with set_new_handler()
		Handles cannot-allocate-memory situation.
		Using this with abort() at end ensures no new() ever comes back NULL.
		Two versions are provided for different environments.
--------------------------------------------------------------------------
*/

void memInstNewHandler (void) {
	sendMsg("OUT OF MEMORY: Attempt to allocate memory failed!");
	memInstReport();
	abort();
	}


int memInstNewHandler (size_t stAllocateSize) {
	char buf[128];
	sprintf(buf, "OUT OF MEMORY: Attempt to allocate %u byte(s) failed!",
		stAllocateSize);
	sendMsg(buf);
	memInstReport();
	abort();
	return 0;
	}


/*
================================================================
NOTES:
  This is a very simple memory instrumentation system.
  It only tracks the total number of memory blocks and bytes.
  (And is therefore useful for catching memory leaks.)

  If this were to be expanded into a more sophisticated system,
  each new/delete could log the exact pointer and byte size,
  possibly with additional information.
================================================================
*/

