/*
==========================================================================
	TABLES.CPP (Version 4.01)
		Source code for standard RPG table-handling utilities.
	Copyright (c) 2001-2008 by Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
==========================================================================
*/

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "tables.h"
#include "dice.h"


// Define table start flag in text file
#define TABLE_FLAG				"Table: "
#define	TABLE_FLAG_LEN		(strlen(TABLE_FLAG))


// Module messaging function
static void sendMsg (const char *msg) { fprintf(stderr, "%s\n", msg); }


/*
---------------------------------------------------------------------
	fgetline() -- reads a line from a file & replaces newline.
---------------------------------------------------------------------
*/

char *fgetline (char *s, int n, FILE *stream)
	{
	if (fgets(s, n, stream))
		{
		int length = strlen(s);
		if (length < n)
			s[length-1] = 0;
		return s;
		}
	return NULL;
	}


/*
---------------------------------------------------------------------
  tabTokPtr() -- point to the Nth tab-delimited token in a string.
    Sequential tabs are treated as a single delimiter.
---------------------------------------------------------------------
*/

const char *tabTokPtr (const char *s, int n)
	{
	for (; *s && n; ++s)
		if (*s == '\t' && *(s+1) != '\t')
			--n;
	return (*s ? s : NULL);
	}


/*
---------------------------------------------------------------------
  rangeMin(), rangeMax() -- parse a string as an integer range.
    Examples: "15-20", "18", or "-".
---------------------------------------------------------------------
*/

int rangeMin (const char *s)
	{
	return (s ? atoi(s) : 0);
	}


int rangeMax (const char *s)
	{
	if (s)
		{
		const char *p = s;
		while (isdigit(*p)) p++;
		return (*p == '-' ? atoi(p+1) : atoi(s));
		}
	return 0;
	}


/*
=====================================================================
  Table class
=====================================================================
*/

// Constructor

Table::Table (const char *name, FILE *f)
	{
	assert(name && f);
  next = NULL;
  rows = new StringList();
  assert(rows);
  rows->appendString(name);
	readRows(f);
	countRows();
	}


// Destructor

Table::~Table (void)
	{
	delete rows;
	delete next;
	}


// Return pointer to the name string

const char *Table::getName (void)
	{
	return rows->getString(0);
	}


// Return the minimal row

int Table::getRowMin (void)
	{
	return rowMin;
	}


// Return the maximum row

int Table::getRowMax (void)
	{
	return rowMax;
	}


// Return maximum value in a column

int Table::getColRangeMax (int col)
  {
  int max = 0;
  for (int i = 1; i <= rowMax; i++) 
    {
    int cellMax = rangeMax(tabTokPtr(getRow(i), col));
    if (cellMax > max)
      max = cellMax;
    }
  return max;
  }


// Return pointer to specified row

const char* Table::getRow (int row)
	{
  if (row < rowMin)
    return NULL;
  else
    return rows->getString(rowIndex(row));
	}


// Return pointer to next linked table

Table* Table::getNextTable (void)
	{
	return next;
	}


// Set pointer to the next linked table

void Table::setNextTable (Table *n)
	{
	assert(next == NULL);
	next = n;
	}


// Lookup row by particular value in column

const char* Table::lookupRow (int lookup, int col)
	{
	assert(lookup >= 0 && col >= 0);
  for (int i = 1; i <= rowMax; i++) {
    const char *row = rows->getString(rowIndex(i));
    if (lookup <= rangeMax(tabTokPtr(row, col)))
      return row;
    }
	return NULL;
	}


// Roll up row by particular column

const char* Table::rollRow (int col)
	{
	assert(col >= 0);
  Dice diceSpec;
  const char *row = rows->getString(rowIndex(-1));
  diceSpec.read(tabTokPtr(row, col));
	return lookupRow(diceSpec.roll(), col);
	}


// Reads rows from open file until blank line

void Table::readRows (FILE *f)
	{
	char buf[maxRowSize];
	while (fgetline(buf, maxRowSize, f))
		{
		if (*buf == 0) break;
    rows->appendString(buf);
		}
	}


// Counts rows to set min, max values

void Table::countRows (void)
	{
	rowMin = rowMax = 0;
	int foundBreak = 0;
  for (int i = 0; i < rows->getNumStrings(); i++)
		{
		if (foundBreak)
			++rowMax;
		else if (!strncmp(rows->getString(i), "--", 2))
			foundBreak = 1;
		else
			--rowMin;
		}
	assert(rowMin <= -1);
	assert(rowMax >= +1);
	}


// Convert row number to index in StringList

int Table::rowIndex(int row) 
  {
  assert(row >= rowMin);
  return row - rowMin;
  }


/*
=====================================================================
	Class TableList -- a list of arbitrary tables.
=====================================================================
*/

// Constructor

TableList::TableList (const char *fname)
	{
	tableFirst = tableLast = NULL;
	if (fname)
		loadFile(fname);
	}


// Destructor

TableList::~TableList (void)
	{
	delete tableFirst;
	}


// Return pointer to a single named table

Table *TableList::getTable (const char *name)
	{
	assert(name);
	for (Table *p = tableFirst; p; p = p->getNextTable())
		if (!strcmp(name, p->getName()))
			return p;
	return NULL;
	}


// Load all tables from a single file

void TableList::loadFile (const char *fname)
	{
	assert(fname);

	// Open the file
	FILE *f = fopen(fname, "r");
	if (!f)
		{
		sendMsg("TableList could not open file for loading.");
		return;
		}

	// Search for tables in file
  char buf[Table::maxRowSize];
	while (fgetline(buf, Table::maxRowSize, f))
		if (!strncmp(buf, TABLE_FLAG, TABLE_FLAG_LEN))
			appendTable(new Table(&buf[TABLE_FLAG_LEN], f));

	// Close the file
	fclose(f);
	}


// Append a table to the end of the list

void TableList::appendTable (Table *table)
	{
	assert(table);

	// Check for a duplicate table name
	if (getTable(table->getName()))
		{
		sendMsg("TableList found duplicate table name; discarding.");
		return;
		}

	// Append the table
	if (!tableFirst)
		tableFirst = tableLast = table;
	else
		{
		assert(tableLast);
		tableLast->setNextTable(table);
		tableLast = table;
		}
	}

