/*
==========================================================================
	TABLES.H (Version 4.01)
		Interface to standard RPG table-handling utilities.
	Copyright (c) 2001-2008 by Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
==========================================================================
*/

#ifndef TABLES_H
#define TABLES_H

#include "strlist.h"  // define append-only StringList


/*
----------------------------------------------------------------
  Table class
----------------------------------------------------------------
*/

class Table
	{
	public:
    // Enumerations
    enum { maxRowSize = 1024 };

		// Constructors
		Table 	    (const char *name, FILE *f);
		~Table 	    (void);

		// Accessors
		const char*	getName (void);
		const char* getRow (int row);
    int					getRowMin (void);
		int					getRowMax (void);
    int         getColRangeMax (int col = 0);

		// Lookups
		const char*	lookupRow (int lookup, int col = 0);
		const char* rollRow (int col = 0);

    // Link to next table
		Table*      getNextTable (void);
		void				setNextTable (Table *next);

	protected:
		void				readRows (FILE *f);
		void				countRows (void);
    int         rowIndex (int row);

	private:
    StringList *rows;
    int rowMin, rowMax;
		Table *next;
	};


/*
----------------------------------------------------------------
  TableList class
----------------------------------------------------------------
*/

class TableList
	{
	public:
		// Constructors
		TableList 	(const char *fname = NULL);
		~TableList 	(void);

		// Accessors
		void				loadFile (const char *fname);
		Table*	    getTable (const char *name);

	protected:
		void				appendTable (Table *table);

	private:
		Table *tableFirst, *tableLast;
	};


#endif


/*
========================================================================
TABLES MODULE DOCUMENTATION

The "Tables" system is meant as an easy way to incorporate standard
RPG-type random tables and charts into any C++ computer program.
The ideal use is to very simply copy a chart directly out of one of the
d20 Open Gaming System Reference Documents ("SRDs"), paste with minimal
alterations into a text file, and have it be usable by this infrastructure.

The TABLES library will manage and access a number of tables of
the appropriate format, read from one or more text files, and their
individual rows. Use the string-tokenization functions in the DRCSTR
library to access individual column entries.

Using the d20 SRDs as a guide, these tables have the following format:
- They begin with the line "Table: <name>" as an identifier.
	No two tables in the same table list may share the same <name>.
- One or more header lines are expected (possibly blank).
- A header break line starting with two dashes ("--") sits after the header.
	(That is, dashes are used as underlining beneath the header.)
- The material rows of the table follow the header break.
- Columns in any line are delimited by tabs. Repeated tabs are treated
	as one delimiter (the 3.0 SRDs do this for formatting niceties).
- Each table ends with a blank line.

I favor labelling table text files with an extension of ".tbl".
All the tables in a single file are read into memory with one call.
Thereafter, particular rows of any table can be retrieved and parsed.

The most common use will be to randomly roll on some table and use
Table::lookupRow() to retrieve the line that the roll matches with. In this
method, any column may have a roll index whose vertical entries will look
like the following: "01-20", "21-25", "26", "-", "27-30", etc.
These are automatically interpreted by Table::lookupRow().
If no column is given, column #0 is assumed as the roll index.

It's also possible to have the dice to roll indicated in the table data
itself, i.e., a Dice string descriptor (see DICE.H/CPP) in the
correct header column, retrieved by getRow(-1), parsed, rolled
by Dice::roll(), and the result retrieved with Table::lookupRow().
========================================================================
*/

